function [alphaout, Aout] = MonocyclicRepresentation (alphain, Ain)

    function [evalues, repeats] = eigvalc(A)

        % eigval  Eigenvalues and their algebraic multiplicity.
        %
        % evalues = eigval(A) returns the distinct eigenvalues of A,
        % with duplicates removed and sorted in increasing order.
        %
        % [evalues, repeats] = eigval(A) also returns the row vector
        % repeats that gives the multiplicity of each eigenvalue.
        % The sum of the multiplicities is n.
        %
        % Examples: Let A = eye(n) and B = diag([3 4]).
        % For A, evalues is 1 and repeats is n.
        % For B, evalues is [4; 3]  and repeats is [1 1].

        tol = sqrt(eps);
        lambda = sort(eig(A));
        lambda = round(lambda/tol) * tol;
        %
        % lambda gives all n eigenvalues (repetitions included).
        %
        evalues = unique(lambda);
        n = length(lambda);
        d = length(evalues);
        A = ones(n, 1) * evalues';
        B = lambda * ones(1, d);
        MATCH = abs(A-B) <= tol;
        %
        % MATCH is an n by d zero matrix except
        % MATCH(i,j) = 1 when lambda(i) = evalues(j).
        % Summing the columns gives the row vector repeats.
        %
        repeats = sum(MATCH);
    end

    function M = devec(v, rows)

        M = [];
        for i=1:length(v)/rows
            M = [M, v((i-1)*rows+1:i*rows)];
        end
    end

    function Vn = eigentrans (A, evalues, repeats)

        V = [];
        L = [];
        for i=1:length(evalues)
            Li = -evalues(i)*(-eye(repeats(i)) + diag(ones(1,repeats(i)-1),1));
            M = kron(eye(repeats(i)),A) - kron(conj(Li'),eye(size(A,1)));
            m = zeros(size(M,1),1);
            % normalization condition
            Mn = kron(eye(repeats(i)), ones(1,size(A,1)));
            mn = ones(repeats(i),1);
            M = [Mn; M];
            m = [mn; m];
            Vi = devec (M \ m, size(A,1));
            V = [V Vi];
        end

        % transform it by an upper toeplitz matrix R such that R*inv(V)=ones
        v = sum(inv(V),2);
        ix = 1;
        G = [];
        for k=1:length(evalues)
            g = zeros(1,repeats(k));
            for i=1:repeats(k)
                s=1;
                for j=1:i-1
                    s = s - g(j) * v(ix+repeats(k)-1-i+j);
                end
                g(i) = s / v(ix+repeats(k)-1);
            end
            Gk = toeplitz([g(1) zeros(1,repeats(k)-1)], g);
            G(ix:ix+repeats(k)-1,ix:ix+repeats(k)-1) = Gk;
            ix = ix + repeats(k);
        end
        Vn=V*inv(G);
    end


    % calculates the feedback Erlang blocks correnponding to the eigenvalues of
    % matrix A.
    % The length of the resulting vector is the number of feedback Erlang blocks
    % used. The resulting vector is ordered according to the dominant eigenvalues of
    % the FEBs.
    % Each element of the vector is a structure with the following fields:
    % - lambda, z, n: properties of the FEB (intensity, fb prob., size)
    % - multip: multiplicity
    % - evals, emuls: eigenvalues of the FEB and their multiplicities
    function febs = generateFEBs (A)

        [evalues, repeats] = eigvalc(A);

        % calculate the parameters of the febs
        febs = [];
        ix = 1;
        i = 1;
        evals = [];
        emuls = [];
        while i<=length(evalues)
            multip = repeats(i);
            if imag(evalues(i)) == 0
                n = 1;
                sigma = -evalues(i);
                z = 0;
                ev = evalues(i);
                em = multip;
            else
                n = 3;
                while imag(evalues(i)) / real(evalues(i)) >= cot(pi/n)
                    n = n + 1;
                end
                sigma = -(2*real(evalues(i)) + imag(evalues(i))*(cot(pi/n) - tan(pi/n))) / 2;
                z = (-imag(evalues(i))*(cot(pi/n) + tan(pi/n)) / (2*sigma))^n;
                ev = []; em = [];
                for k=1:n
                    ev = [ev; complex(-(1-z^(1/n)*cos(2*(k-1)*pi/n))*sigma,z^(1/n)*sin(2*(k-1)*pi/n)*sigma)];
                    em = [em; multip];
                end
            end

            febs(ix).lambda = sigma;
            febs(ix).z = z;
            febs(ix).n = n;
            febs(ix).multip = multip;
            febs(ix).evals = ev;
            febs(ix).emuls = em;

            if imag(evalues(i)) == 0
                i = i + 1;
            else
                i = i + 2;
            end
            ix = ix + 1;
        end

        % ordering according to the dominant eigenvalue (simple bubble sort)
        N = length(febs);
        for i=1:N-1
            for j=1:N-i
                if max(febs(j).evals) < max(febs(j+1).evals)
                    F = febs(j+1);
                    febs(j+1) = febs(j);
                    febs(j) = F;
                end
            end
        end
    end

    function A = febGenerator (lambda, z, n, multip)

        A = zeros(multip*n);
        for fi=1:multip
            lv = ones(1,n)*lambda;
            A((fi-1)*n+1:fi*n, (fi-1)*n+1:fi*n) = -diag(lv) + diag(lv(1:n-1),1);
            A(fi*n,(fi-1)*n+1) = A(fi*n,(fi-1)*n+1) + z * lambda;
            if fi<multip
                A(fi*n,fi*n+1) = (1-z)*lambda;
            end
        end
    end


% quasi diagonalization of matrix A
[evalues, repeats] = eigvalc(Ain);
Vn = eigentrans (Ain, evalues, repeats);

% assemble generator matrix of the hyper-feb 
febs = generateFEBs (Ain);
FEB = [];
pos = 1;
devals = [];
demuls = [];
for i=1:length(febs)
    Ni = febs(i).n*febs(i).multip;
    M = febGenerator (febs(i).lambda, febs(i).z, febs(i).n, febs(i).multip);
    FEB(pos:pos+Ni-1,pos:pos+Ni-1) = M;
    if i<length(febs)
        FEB(pos+Ni-1, pos+Ni) = -sum(FEB(pos+Ni-1,:));
    end
    devals = [devals; febs(i).evals];
    demuls = [demuls; febs(i).emuls];
    pos = pos + Ni;
end

evals = [];
emuls = [];
for i=1:length(devals)
    evals = [evals; ones(demuls(i),1)*devals(i)];
    emuls = [emuls; (demuls(i):-1:1)'];
end

V1 = eigentrans (FEB, devals, demuls);

pos = 1;
i = 1;
P = zeros(size(Ain,1),size(FEB,2));
while i<=length(evalues)
    multip = repeats(i);

    [tmp, pix1] = min ( (evals-evalues(i)).^2);
    for j=0:multip-1
        P(pos+j,pix1+j) = 1;
    end
    
    if imag(evalues(i)) ~= 0
        [tmp, pix2] = min ( (evals-conj(evalues(i))).^2);
        for j=0:multip-1
            P(pos+multip+j,pix2+j) = 1;
        end
        pos = pos + multip;
        i = i + 1;
    end
    pos = pos + multip;
    i = i + 1;
end
V2 = Vn*P;

Aout = FEB;
alphaout = alphain * V2 / V1;
end